/*
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
/*
 * Warning! This file is generated. Modify at your own risk.
 */

package com.google.api.services.dfareporting.model;

import com.google.api.client.json.GenericJson;
import com.google.api.client.json.JsonString;
import com.google.api.client.util.DateTime;
import com.google.common.primitives.UnsignedLong;

/**
 * Represents a Report resource.
 *
 * <p> This is the Java data model class that specifies how to parse/serialize into the JSON that is
 * transmitted over HTTP when working with the DFA Reporting API. For a detailed explanation see:
 * <a href="http://code.google.com/p/google-api-java-client/wiki/Json">http://code.google.com/p/google-api-java-client/wiki/Json</a>
 * </p>
 *
 * <p>
 * Upgrade warning: starting with version 1.12 {@code getResponseHeaders()} is removed, instead use
 * {@link com.google.api.client.http.json.JsonHttpRequest#getLastResponseHeaders()}
 * </p>
 *
 * @author Google, Inc.
 */
@SuppressWarnings("javadoc")
public final class Report extends GenericJson {

  /**
   * The account ID to which this report belongs.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key @JsonString
  private Long accountId;

  /**
   * The report criteria for a report of type "ACTIVE_GRP".
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private ActiveGrpCriteria activeGrpCriteria;

  /**
   * The report criteria for a report of type "STANDARD".
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private Criteria criteria;

  /**
   * The report criteria for a report of type "CROSS_DIMENSION_REACH".
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private CrossDimensionReachCriteria crossDimensionReachCriteria;

  /**
   * The report's email delivery settings.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private Delivery delivery;

  /**
   * The eTag of this response for caching purposes.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String etag;

  /**
   * The file name used when generating report files for this report.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String fileName;

  /**
   * The report criteria for a report of type "FLOODLIGHT".
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private FloodlightCriteria floodlightCriteria;

  /**
   * The output format of the report, one of: - "CSV"  - "EXCEL"  If not specified, default format
   * is "CSV". Note that the actual format in the completed report file might differ if for instance
   * the report's size exceeds the format's capabilities. "CSV" will then be the fallback format.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String format;

  /**
   * The unique ID identifying this report resource.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key @JsonString
  private Long id;

  /**
   * The kind of resource this is, in this case dfareporting#report.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String kind;

  /**
   * The timestamp (in milliseconds since epoch) of when this report was last modified.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key @JsonString
  private UnsignedLong lastModifiedTime;

  /**
   * The name of the report.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String name;

  /**
   * The user profile id of the owner of this report.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key @JsonString
  private Long ownerProfileId;

  /**
   * The report criteria for a report of type "PATH_TO_CONVERSION".
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private PathToConversionCriteria pathToConversionCriteria;

  /**
   * The report criteria for a report of type "REACH".
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private ReachCriteria reachCriteria;

  /**
   * The report's schedule. Can only be set if the report's 'dateRange' is a relative date range and
   * the relative date range is not "TODAY".
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private Schedule schedule;

  /**
   * The subbaccount ID to which this report belongs if applicable.
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key @JsonString
  private Long subAccountId;

  /**
   * The type of the report, one of: - STANDARD  - REACH  - ACTIVE_GRP  - PATH_TO_CONVERSION  -
   * FLOODLIGHT  - CROSS_DIMENSION_REACH
   * The value may be {@code null}.
   */
  @com.google.api.client.util.Key
  private String type;

  /**
   * The account ID to which this report belongs.
   * The value returned may be {@code null}.
   */
  public Long getAccountId() {
    return accountId;
  }

  /**
   * The account ID to which this report belongs.
   * The value set may be {@code null}.
   */
  public Report setAccountId(Long accountId) {
    this.accountId = accountId;
    return this;
  }

  /**
   * The report criteria for a report of type "ACTIVE_GRP".
   * The value returned may be {@code null}.
   */
  public ActiveGrpCriteria getActiveGrpCriteria() {
    return activeGrpCriteria;
  }

  /**
   * The report criteria for a report of type "ACTIVE_GRP".
   * The value set may be {@code null}.
   */
  public Report setActiveGrpCriteria(ActiveGrpCriteria activeGrpCriteria) {
    this.activeGrpCriteria = activeGrpCriteria;
    return this;
  }

  /**
   * The report criteria for a report of type "STANDARD".
   * The value returned may be {@code null}.
   */
  public Criteria getCriteria() {
    return criteria;
  }

  /**
   * The report criteria for a report of type "STANDARD".
   * The value set may be {@code null}.
   */
  public Report setCriteria(Criteria criteria) {
    this.criteria = criteria;
    return this;
  }

  /**
   * The report criteria for a report of type "CROSS_DIMENSION_REACH".
   * The value returned may be {@code null}.
   */
  public CrossDimensionReachCriteria getCrossDimensionReachCriteria() {
    return crossDimensionReachCriteria;
  }

  /**
   * The report criteria for a report of type "CROSS_DIMENSION_REACH".
   * The value set may be {@code null}.
   */
  public Report setCrossDimensionReachCriteria(CrossDimensionReachCriteria crossDimensionReachCriteria) {
    this.crossDimensionReachCriteria = crossDimensionReachCriteria;
    return this;
  }

  /**
   * The report's email delivery settings.
   * The value returned may be {@code null}.
   */
  public Delivery getDelivery() {
    return delivery;
  }

  /**
   * The report's email delivery settings.
   * The value set may be {@code null}.
   */
  public Report setDelivery(Delivery delivery) {
    this.delivery = delivery;
    return this;
  }

  /**
   * The eTag of this response for caching purposes.
   * The value returned may be {@code null}.
   */
  public String getEtag() {
    return etag;
  }

  /**
   * The eTag of this response for caching purposes.
   * The value set may be {@code null}.
   */
  public Report setEtag(String etag) {
    this.etag = etag;
    return this;
  }

  /**
   * The file name used when generating report files for this report.
   * The value returned may be {@code null}.
   */
  public String getFileName() {
    return fileName;
  }

  /**
   * The file name used when generating report files for this report.
   * The value set may be {@code null}.
   */
  public Report setFileName(String fileName) {
    this.fileName = fileName;
    return this;
  }

  /**
   * The report criteria for a report of type "FLOODLIGHT".
   * The value returned may be {@code null}.
   */
  public FloodlightCriteria getFloodlightCriteria() {
    return floodlightCriteria;
  }

  /**
   * The report criteria for a report of type "FLOODLIGHT".
   * The value set may be {@code null}.
   */
  public Report setFloodlightCriteria(FloodlightCriteria floodlightCriteria) {
    this.floodlightCriteria = floodlightCriteria;
    return this;
  }

  /**
   * The output format of the report, one of: - "CSV"  - "EXCEL"  If not specified, default format
   * is "CSV". Note that the actual format in the completed report file might differ if for instance
   * the report's size exceeds the format's capabilities. "CSV" will then be the fallback format.
   * The value returned may be {@code null}.
   */
  public String getFormat() {
    return format;
  }

  /**
   * The output format of the report, one of: - "CSV"  - "EXCEL"  If not specified, default format
   * is "CSV". Note that the actual format in the completed report file might differ if for instance
   * the report's size exceeds the format's capabilities. "CSV" will then be the fallback format.
   * The value set may be {@code null}.
   */
  public Report setFormat(String format) {
    this.format = format;
    return this;
  }

  /**
   * The unique ID identifying this report resource.
   * The value returned may be {@code null}.
   */
  public Long getId() {
    return id;
  }

  /**
   * The unique ID identifying this report resource.
   * The value set may be {@code null}.
   */
  public Report setId(Long id) {
    this.id = id;
    return this;
  }

  /**
   * The kind of resource this is, in this case dfareporting#report.
   * The value returned may be {@code null}.
   */
  public String getKind() {
    return kind;
  }

  /**
   * The kind of resource this is, in this case dfareporting#report.
   * The value set may be {@code null}.
   */
  public Report setKind(String kind) {
    this.kind = kind;
    return this;
  }

  /**
   * The timestamp (in milliseconds since epoch) of when this report was last modified.
   * The value returned may be {@code null}.
   */
  public UnsignedLong getLastModifiedTime() {
    return lastModifiedTime;
  }

  /**
   * The timestamp (in milliseconds since epoch) of when this report was last modified.
   * The value set may be {@code null}.
   */
  public Report setLastModifiedTime(UnsignedLong lastModifiedTime) {
    this.lastModifiedTime = lastModifiedTime;
    return this;
  }

  /**
   * The name of the report.
   * The value returned may be {@code null}.
   */
  public String getName() {
    return name;
  }

  /**
   * The name of the report.
   * The value set may be {@code null}.
   */
  public Report setName(String name) {
    this.name = name;
    return this;
  }

  /**
   * The user profile id of the owner of this report.
   * The value returned may be {@code null}.
   */
  public Long getOwnerProfileId() {
    return ownerProfileId;
  }

  /**
   * The user profile id of the owner of this report.
   * The value set may be {@code null}.
   */
  public Report setOwnerProfileId(Long ownerProfileId) {
    this.ownerProfileId = ownerProfileId;
    return this;
  }

  /**
   * The report criteria for a report of type "PATH_TO_CONVERSION".
   * The value returned may be {@code null}.
   */
  public PathToConversionCriteria getPathToConversionCriteria() {
    return pathToConversionCriteria;
  }

  /**
   * The report criteria for a report of type "PATH_TO_CONVERSION".
   * The value set may be {@code null}.
   */
  public Report setPathToConversionCriteria(PathToConversionCriteria pathToConversionCriteria) {
    this.pathToConversionCriteria = pathToConversionCriteria;
    return this;
  }

  /**
   * The report criteria for a report of type "REACH".
   * The value returned may be {@code null}.
   */
  public ReachCriteria getReachCriteria() {
    return reachCriteria;
  }

  /**
   * The report criteria for a report of type "REACH".
   * The value set may be {@code null}.
   */
  public Report setReachCriteria(ReachCriteria reachCriteria) {
    this.reachCriteria = reachCriteria;
    return this;
  }

  /**
   * The report's schedule. Can only be set if the report's 'dateRange' is a relative date range and
   * the relative date range is not "TODAY".
   * The value returned may be {@code null}.
   */
  public Schedule getSchedule() {
    return schedule;
  }

  /**
   * The report's schedule. Can only be set if the report's 'dateRange' is a relative date range and
   * the relative date range is not "TODAY".
   * The value set may be {@code null}.
   */
  public Report setSchedule(Schedule schedule) {
    this.schedule = schedule;
    return this;
  }

  /**
   * The subbaccount ID to which this report belongs if applicable.
   * The value returned may be {@code null}.
   */
  public Long getSubAccountId() {
    return subAccountId;
  }

  /**
   * The subbaccount ID to which this report belongs if applicable.
   * The value set may be {@code null}.
   */
  public Report setSubAccountId(Long subAccountId) {
    this.subAccountId = subAccountId;
    return this;
  }

  /**
   * The type of the report, one of: - STANDARD  - REACH  - ACTIVE_GRP  - PATH_TO_CONVERSION  -
   * FLOODLIGHT  - CROSS_DIMENSION_REACH
   * The value returned may be {@code null}.
   */
  public String getType() {
    return type;
  }

  /**
   * The type of the report, one of: - STANDARD  - REACH  - ACTIVE_GRP  - PATH_TO_CONVERSION  -
   * FLOODLIGHT  - CROSS_DIMENSION_REACH
   * The value set may be {@code null}.
   */
  public Report setType(String type) {
    this.type = type;
    return this;
  }

  /**
   * The report criteria for a report of type "ACTIVE_GRP".
   */
  public static final class ActiveGrpCriteria extends GenericJson {

    /**
     * The date range this report should be run for.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private DateRange dateRange;

    /**
     * The list of filters on which dimensions are filtered. Filters for different dimensions are
     * ANDed, filters for the same dimension are grouped together and ORed. A valid active GRP report
     * needs to have exactly one DimensionValue for the United States in addition to any advertiser or
     * campaign dimension values.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private java.util.List<DimensionValue> dimensionFilters;

    static {
      // hack to force ProGuard to consider DimensionValue used, since otherwise it would be stripped out
      // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
      com.google.api.client.util.Data.nullOf(DimensionValue.class);
    }

    /**
     * The list of dimensions the report should include.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private java.util.List<SortedDimension> dimensions;

    static {
      // hack to force ProGuard to consider SortedDimension used, since otherwise it would be stripped out
      // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
      com.google.api.client.util.Data.nullOf(SortedDimension.class);
    }

    /**
     * The list of names of metrics the report should include.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private java.util.List<String> metricNames;

    /**
     * The date range this report should be run for.
     * The value returned may be {@code null}.
     */
    public DateRange getDateRange() {
      return dateRange;
    }

    /**
     * The date range this report should be run for.
     * The value set may be {@code null}.
     */
    public ActiveGrpCriteria setDateRange(DateRange dateRange) {
      this.dateRange = dateRange;
      return this;
    }

    /**
     * The list of filters on which dimensions are filtered. Filters for different dimensions are
     * ANDed, filters for the same dimension are grouped together and ORed. A valid active GRP report
     * needs to have exactly one DimensionValue for the United States in addition to any advertiser or
     * campaign dimension values.
     * The value returned may be {@code null}.
     */
    public java.util.List<DimensionValue> getDimensionFilters() {
      return dimensionFilters;
    }

    /**
     * The list of filters on which dimensions are filtered. Filters for different dimensions are
     * ANDed, filters for the same dimension are grouped together and ORed. A valid active GRP report
     * needs to have exactly one DimensionValue for the United States in addition to any advertiser or
     * campaign dimension values.
     * The value set may be {@code null}.
     */
    public ActiveGrpCriteria setDimensionFilters(java.util.List<DimensionValue> dimensionFilters) {
      this.dimensionFilters = dimensionFilters;
      return this;
    }

    /**
     * The list of dimensions the report should include.
     * The value returned may be {@code null}.
     */
    public java.util.List<SortedDimension> getDimensions() {
      return dimensions;
    }

    /**
     * The list of dimensions the report should include.
     * The value set may be {@code null}.
     */
    public ActiveGrpCriteria setDimensions(java.util.List<SortedDimension> dimensions) {
      this.dimensions = dimensions;
      return this;
    }

    /**
     * The list of names of metrics the report should include.
     * The value returned may be {@code null}.
     */
    public java.util.List<String> getMetricNames() {
      return metricNames;
    }

    /**
     * The list of names of metrics the report should include.
     * The value set may be {@code null}.
     */
    public ActiveGrpCriteria setMetricNames(java.util.List<String> metricNames) {
      this.metricNames = metricNames;
      return this;
    }

  }

  /**
   * The report criteria for a report of type "STANDARD".
   */
  public static final class Criteria extends GenericJson {

    /**
     * Activity group.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private Activities activities;

    /**
     * Custom Rich Media Events group.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private CustomRichMediaEvents customRichMediaEvents;

    /**
     * The date range for which this report should be run.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private DateRange dateRange;

    /**
     * The list of filters on which dimensions are filtered. Filters for different dimensions are
     * ANDed, filters for the same dimension are grouped together and ORed.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private java.util.List<DimensionValue> dimensionFilters;

    static {
      // hack to force ProGuard to consider DimensionValue used, since otherwise it would be stripped out
      // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
      com.google.api.client.util.Data.nullOf(DimensionValue.class);
    }

    /**
     * The list of standard dimensions the report should include.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private java.util.List<SortedDimension> dimensions;

    static {
      // hack to force ProGuard to consider SortedDimension used, since otherwise it would be stripped out
      // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
      com.google.api.client.util.Data.nullOf(SortedDimension.class);
    }

    /**
     * The list of names of metrics the report should include.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private java.util.List<String> metricNames;

    /**
     * Activity group.
     * The value returned may be {@code null}.
     */
    public Activities getActivities() {
      return activities;
    }

    /**
     * Activity group.
     * The value set may be {@code null}.
     */
    public Criteria setActivities(Activities activities) {
      this.activities = activities;
      return this;
    }

    /**
     * Custom Rich Media Events group.
     * The value returned may be {@code null}.
     */
    public CustomRichMediaEvents getCustomRichMediaEvents() {
      return customRichMediaEvents;
    }

    /**
     * Custom Rich Media Events group.
     * The value set may be {@code null}.
     */
    public Criteria setCustomRichMediaEvents(CustomRichMediaEvents customRichMediaEvents) {
      this.customRichMediaEvents = customRichMediaEvents;
      return this;
    }

    /**
     * The date range for which this report should be run.
     * The value returned may be {@code null}.
     */
    public DateRange getDateRange() {
      return dateRange;
    }

    /**
     * The date range for which this report should be run.
     * The value set may be {@code null}.
     */
    public Criteria setDateRange(DateRange dateRange) {
      this.dateRange = dateRange;
      return this;
    }

    /**
     * The list of filters on which dimensions are filtered. Filters for different dimensions are
     * ANDed, filters for the same dimension are grouped together and ORed.
     * The value returned may be {@code null}.
     */
    public java.util.List<DimensionValue> getDimensionFilters() {
      return dimensionFilters;
    }

    /**
     * The list of filters on which dimensions are filtered. Filters for different dimensions are
     * ANDed, filters for the same dimension are grouped together and ORed.
     * The value set may be {@code null}.
     */
    public Criteria setDimensionFilters(java.util.List<DimensionValue> dimensionFilters) {
      this.dimensionFilters = dimensionFilters;
      return this;
    }

    /**
     * The list of standard dimensions the report should include.
     * The value returned may be {@code null}.
     */
    public java.util.List<SortedDimension> getDimensions() {
      return dimensions;
    }

    /**
     * The list of standard dimensions the report should include.
     * The value set may be {@code null}.
     */
    public Criteria setDimensions(java.util.List<SortedDimension> dimensions) {
      this.dimensions = dimensions;
      return this;
    }

    /**
     * The list of names of metrics the report should include.
     * The value returned may be {@code null}.
     */
    public java.util.List<String> getMetricNames() {
      return metricNames;
    }

    /**
     * The list of names of metrics the report should include.
     * The value set may be {@code null}.
     */
    public Criteria setMetricNames(java.util.List<String> metricNames) {
      this.metricNames = metricNames;
      return this;
    }

  }

  /**
   * The report criteria for a report of type "CROSS_DIMENSION_REACH".
   */
  public static final class CrossDimensionReachCriteria extends GenericJson {

    /**
     * The list of dimensions the report should include.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private java.util.List<SortedDimension> breakdown;

    static {
      // hack to force ProGuard to consider SortedDimension used, since otherwise it would be stripped out
      // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
      com.google.api.client.util.Data.nullOf(SortedDimension.class);
    }

    /**
     * The date range this report should be run for.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private DateRange dateRange;

    /**
     * The dimension option, one of: - "ADVERTISER"  - "CAMPAIGN"  - "SITE_BY_ADVERTISER"  -
     * "SITE_BY_CAMPAIGN"
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String dimension;

    /**
     * The list of filters on which dimensions are filtered.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private java.util.List<DimensionValue> dimensionFilters;

    static {
      // hack to force ProGuard to consider DimensionValue used, since otherwise it would be stripped out
      // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
      com.google.api.client.util.Data.nullOf(DimensionValue.class);
    }

    /**
     * The list of names of metrics the report should include.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private java.util.List<String> metricNames;

    /**
     * The list of names of overlap metrics the report should include.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private java.util.List<String> overlapMetricNames;

    /**
     * Whether the report is pivoted or not. Defaults to true.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private Boolean pivoted;

    /**
     * The list of dimensions the report should include.
     * The value returned may be {@code null}.
     */
    public java.util.List<SortedDimension> getBreakdown() {
      return breakdown;
    }

    /**
     * The list of dimensions the report should include.
     * The value set may be {@code null}.
     */
    public CrossDimensionReachCriteria setBreakdown(java.util.List<SortedDimension> breakdown) {
      this.breakdown = breakdown;
      return this;
    }

    /**
     * The date range this report should be run for.
     * The value returned may be {@code null}.
     */
    public DateRange getDateRange() {
      return dateRange;
    }

    /**
     * The date range this report should be run for.
     * The value set may be {@code null}.
     */
    public CrossDimensionReachCriteria setDateRange(DateRange dateRange) {
      this.dateRange = dateRange;
      return this;
    }

    /**
     * The dimension option, one of: - "ADVERTISER"  - "CAMPAIGN"  - "SITE_BY_ADVERTISER"  -
     * "SITE_BY_CAMPAIGN"
     * The value returned may be {@code null}.
     */
    public String getDimension() {
      return dimension;
    }

    /**
     * The dimension option, one of: - "ADVERTISER"  - "CAMPAIGN"  - "SITE_BY_ADVERTISER"  -
     * "SITE_BY_CAMPAIGN"
     * The value set may be {@code null}.
     */
    public CrossDimensionReachCriteria setDimension(String dimension) {
      this.dimension = dimension;
      return this;
    }

    /**
     * The list of filters on which dimensions are filtered.
     * The value returned may be {@code null}.
     */
    public java.util.List<DimensionValue> getDimensionFilters() {
      return dimensionFilters;
    }

    /**
     * The list of filters on which dimensions are filtered.
     * The value set may be {@code null}.
     */
    public CrossDimensionReachCriteria setDimensionFilters(java.util.List<DimensionValue> dimensionFilters) {
      this.dimensionFilters = dimensionFilters;
      return this;
    }

    /**
     * The list of names of metrics the report should include.
     * The value returned may be {@code null}.
     */
    public java.util.List<String> getMetricNames() {
      return metricNames;
    }

    /**
     * The list of names of metrics the report should include.
     * The value set may be {@code null}.
     */
    public CrossDimensionReachCriteria setMetricNames(java.util.List<String> metricNames) {
      this.metricNames = metricNames;
      return this;
    }

    /**
     * The list of names of overlap metrics the report should include.
     * The value returned may be {@code null}.
     */
    public java.util.List<String> getOverlapMetricNames() {
      return overlapMetricNames;
    }

    /**
     * The list of names of overlap metrics the report should include.
     * The value set may be {@code null}.
     */
    public CrossDimensionReachCriteria setOverlapMetricNames(java.util.List<String> overlapMetricNames) {
      this.overlapMetricNames = overlapMetricNames;
      return this;
    }

    /**
     * Whether the report is pivoted or not. Defaults to true.
     * The value returned may be {@code null}.
     */
    public Boolean getPivoted() {
      return pivoted;
    }

    /**
     * Whether the report is pivoted or not. Defaults to true.
     * The value set may be {@code null}.
     */
    public CrossDimensionReachCriteria setPivoted(Boolean pivoted) {
      this.pivoted = pivoted;
      return this;
    }

  }

  /**
   * The report's email delivery settings.
   */
  public static final class Delivery extends GenericJson {

    /**
     * Whether the report should be emailed to the report owner.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private Boolean emailOwner;

    /**
     * The type of delivery for the owner to receive, if enabled. One of: - "ATTACHMENT"  - "LINK"
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String emailOwnerDeliveryType;

    /**
     * The message to be sent with each email.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String message;

    /**
     * The list of recipients to which to email the report.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private java.util.List<Recipient> recipients;

    static {
      // hack to force ProGuard to consider Recipient used, since otherwise it would be stripped out
      // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
      com.google.api.client.util.Data.nullOf(Recipient.class);
    }

    /**
     * Whether the report should be emailed to the report owner.
     * The value returned may be {@code null}.
     */
    public Boolean getEmailOwner() {
      return emailOwner;
    }

    /**
     * Whether the report should be emailed to the report owner.
     * The value set may be {@code null}.
     */
    public Delivery setEmailOwner(Boolean emailOwner) {
      this.emailOwner = emailOwner;
      return this;
    }

    /**
     * The type of delivery for the owner to receive, if enabled. One of: - "ATTACHMENT"  - "LINK"
     * The value returned may be {@code null}.
     */
    public String getEmailOwnerDeliveryType() {
      return emailOwnerDeliveryType;
    }

    /**
     * The type of delivery for the owner to receive, if enabled. One of: - "ATTACHMENT"  - "LINK"
     * The value set may be {@code null}.
     */
    public Delivery setEmailOwnerDeliveryType(String emailOwnerDeliveryType) {
      this.emailOwnerDeliveryType = emailOwnerDeliveryType;
      return this;
    }

    /**
     * The message to be sent with each email.
     * The value returned may be {@code null}.
     */
    public String getMessage() {
      return message;
    }

    /**
     * The message to be sent with each email.
     * The value set may be {@code null}.
     */
    public Delivery setMessage(String message) {
      this.message = message;
      return this;
    }

    /**
     * The list of recipients to which to email the report.
     * The value returned may be {@code null}.
     */
    public java.util.List<Recipient> getRecipients() {
      return recipients;
    }

    /**
     * The list of recipients to which to email the report.
     * The value set may be {@code null}.
     */
    public Delivery setRecipients(java.util.List<Recipient> recipients) {
      this.recipients = recipients;
      return this;
    }

  }

  /**
   * The report criteria for a report of type "FLOODLIGHT".
   */
  public static final class FloodlightCriteria extends GenericJson {

    /**
     * The date range this report should be run for.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private DateRange dateRange;

    /**
     * The list of filters on which dimensions are filtered. Filters for different dimensions are
     * ANDed, filters for the same dimension are grouped together and ORed.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private java.util.List<DimensionValue> dimensionFilters;

    static {
      // hack to force ProGuard to consider DimensionValue used, since otherwise it would be stripped out
      // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
      com.google.api.client.util.Data.nullOf(DimensionValue.class);
    }

    /**
     * The list of dimensions the report should include.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private java.util.List<SortedDimension> dimensions;

    static {
      // hack to force ProGuard to consider SortedDimension used, since otherwise it would be stripped out
      // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
      com.google.api.client.util.Data.nullOf(SortedDimension.class);
    }

    /**
     * The floodlight ID for which to show data in this report. All advertisers associated with that
     * ID will automatically be added. The dimension of the value needs to be
     * 'dfa:floodlightConfigId'.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private DimensionValue floodlightConfigId;

    /**
     * The list of names of metrics the report should include.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private java.util.List<String> metricNames;

    /**
     * The properties of the report.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private ReportProperties reportProperties;

    /**
     * The date range this report should be run for.
     * The value returned may be {@code null}.
     */
    public DateRange getDateRange() {
      return dateRange;
    }

    /**
     * The date range this report should be run for.
     * The value set may be {@code null}.
     */
    public FloodlightCriteria setDateRange(DateRange dateRange) {
      this.dateRange = dateRange;
      return this;
    }

    /**
     * The list of filters on which dimensions are filtered. Filters for different dimensions are
     * ANDed, filters for the same dimension are grouped together and ORed.
     * The value returned may be {@code null}.
     */
    public java.util.List<DimensionValue> getDimensionFilters() {
      return dimensionFilters;
    }

    /**
     * The list of filters on which dimensions are filtered. Filters for different dimensions are
     * ANDed, filters for the same dimension are grouped together and ORed.
     * The value set may be {@code null}.
     */
    public FloodlightCriteria setDimensionFilters(java.util.List<DimensionValue> dimensionFilters) {
      this.dimensionFilters = dimensionFilters;
      return this;
    }

    /**
     * The list of dimensions the report should include.
     * The value returned may be {@code null}.
     */
    public java.util.List<SortedDimension> getDimensions() {
      return dimensions;
    }

    /**
     * The list of dimensions the report should include.
     * The value set may be {@code null}.
     */
    public FloodlightCriteria setDimensions(java.util.List<SortedDimension> dimensions) {
      this.dimensions = dimensions;
      return this;
    }

    /**
     * The floodlight ID for which to show data in this report. All advertisers associated with that
     * ID will automatically be added. The dimension of the value needs to be
     * 'dfa:floodlightConfigId'.
     * The value returned may be {@code null}.
     */
    public DimensionValue getFloodlightConfigId() {
      return floodlightConfigId;
    }

    /**
     * The floodlight ID for which to show data in this report. All advertisers associated with that
     * ID will automatically be added. The dimension of the value needs to be
     * 'dfa:floodlightConfigId'.
     * The value set may be {@code null}.
     */
    public FloodlightCriteria setFloodlightConfigId(DimensionValue floodlightConfigId) {
      this.floodlightConfigId = floodlightConfigId;
      return this;
    }

    /**
     * The list of names of metrics the report should include.
     * The value returned may be {@code null}.
     */
    public java.util.List<String> getMetricNames() {
      return metricNames;
    }

    /**
     * The list of names of metrics the report should include.
     * The value set may be {@code null}.
     */
    public FloodlightCriteria setMetricNames(java.util.List<String> metricNames) {
      this.metricNames = metricNames;
      return this;
    }

    /**
     * The properties of the report.
     * The value returned may be {@code null}.
     */
    public ReportProperties getReportProperties() {
      return reportProperties;
    }

    /**
     * The properties of the report.
     * The value set may be {@code null}.
     */
    public FloodlightCriteria setReportProperties(ReportProperties reportProperties) {
      this.reportProperties = reportProperties;
      return this;
    }

    /**
     * The properties of the report.
     */
    public static final class ReportProperties extends GenericJson {

      /**
       * Include conversions that have no cookie, but do have an exposure path.
       * The value may be {@code null}.
       */
      @com.google.api.client.util.Key
      private Boolean includeAttributedIPConversions;

      /**
       * Include conversions of users with a DoubleClick cookie but without an exposure. That means the
       * user did not click or see an ad from the advertiser within the Floodlight group, or that the
       * interaction happened outside the lookback window.
       * The value may be {@code null}.
       */
      @com.google.api.client.util.Key
      private Boolean includeUnattributedCookieConversions;

      /**
       * Include conversions that have no associated cookies and no exposures. It’s therefore impossible
       * to know how the user was exposed to your ads during the lookback window prior to a conversion.
       * The value may be {@code null}.
       */
      @com.google.api.client.util.Key
      private Boolean includeUnattributedIPConversions;

      /**
       * Include conversions that have no cookie, but do have an exposure path.
       * The value returned may be {@code null}.
       */
      public Boolean getIncludeAttributedIPConversions() {
        return includeAttributedIPConversions;
      }

      /**
       * Include conversions that have no cookie, but do have an exposure path.
       * The value set may be {@code null}.
       */
      public ReportProperties setIncludeAttributedIPConversions(Boolean includeAttributedIPConversions) {
        this.includeAttributedIPConversions = includeAttributedIPConversions;
        return this;
      }

      /**
       * Include conversions of users with a DoubleClick cookie but without an exposure. That means the
       * user did not click or see an ad from the advertiser within the Floodlight group, or that the
       * interaction happened outside the lookback window.
       * The value returned may be {@code null}.
       */
      public Boolean getIncludeUnattributedCookieConversions() {
        return includeUnattributedCookieConversions;
      }

      /**
       * Include conversions of users with a DoubleClick cookie but without an exposure. That means the
       * user did not click or see an ad from the advertiser within the Floodlight group, or that the
       * interaction happened outside the lookback window.
       * The value set may be {@code null}.
       */
      public ReportProperties setIncludeUnattributedCookieConversions(Boolean includeUnattributedCookieConversions) {
        this.includeUnattributedCookieConversions = includeUnattributedCookieConversions;
        return this;
      }

      /**
       * Include conversions that have no associated cookies and no exposures. It’s therefore impossible
       * to know how the user was exposed to your ads during the lookback window prior to a conversion.
       * The value returned may be {@code null}.
       */
      public Boolean getIncludeUnattributedIPConversions() {
        return includeUnattributedIPConversions;
      }

      /**
       * Include conversions that have no associated cookies and no exposures. It’s therefore impossible
       * to know how the user was exposed to your ads during the lookback window prior to a conversion.
       * The value set may be {@code null}.
       */
      public ReportProperties setIncludeUnattributedIPConversions(Boolean includeUnattributedIPConversions) {
        this.includeUnattributedIPConversions = includeUnattributedIPConversions;
        return this;
      }

    }
  }

  /**
   * The report criteria for a report of type "PATH_TO_CONVERSION".
   */
  public static final class PathToConversionCriteria extends GenericJson {

    /**
     * The list of 'dfa:activity' values to filter on.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private java.util.List<DimensionValue> activityFilters;

    static {
      // hack to force ProGuard to consider DimensionValue used, since otherwise it would be stripped out
      // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
      com.google.api.client.util.Data.nullOf(DimensionValue.class);
    }

    /**
     * The list of conversion dimensions the report should include.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private java.util.List<SortedDimension> conversionDimensions;

    static {
      // hack to force ProGuard to consider SortedDimension used, since otherwise it would be stripped out
      // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
      com.google.api.client.util.Data.nullOf(SortedDimension.class);
    }

    /**
     * The list of custom floodlight variables the report should include.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private java.util.List<SortedDimension> customFloodlightVariables;

    static {
      // hack to force ProGuard to consider SortedDimension used, since otherwise it would be stripped out
      // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
      com.google.api.client.util.Data.nullOf(SortedDimension.class);
    }

    /**
     * The date range this report should be run for.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private DateRange dateRange;

    /**
     * The floodlight ID for which to show data in this report. All advertisers associated with that
     * ID will automatically be added. The dimension of the value needs to be
     * 'dfa:floodlightConfigId'.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private DimensionValue floodlightConfigId;

    /**
     * The list of names of metrics the report should include.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private java.util.List<String> metricNames;

    /**
     * The list of per interaction dimensions the report should include.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private java.util.List<SortedDimension> perInteractionDimensions;

    static {
      // hack to force ProGuard to consider SortedDimension used, since otherwise it would be stripped out
      // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
      com.google.api.client.util.Data.nullOf(SortedDimension.class);
    }

    /**
     * The properties of the report.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private ReportProperties reportProperties;

    /**
     * The list of 'dfa:activity' values to filter on.
     * The value returned may be {@code null}.
     */
    public java.util.List<DimensionValue> getActivityFilters() {
      return activityFilters;
    }

    /**
     * The list of 'dfa:activity' values to filter on.
     * The value set may be {@code null}.
     */
    public PathToConversionCriteria setActivityFilters(java.util.List<DimensionValue> activityFilters) {
      this.activityFilters = activityFilters;
      return this;
    }

    /**
     * The list of conversion dimensions the report should include.
     * The value returned may be {@code null}.
     */
    public java.util.List<SortedDimension> getConversionDimensions() {
      return conversionDimensions;
    }

    /**
     * The list of conversion dimensions the report should include.
     * The value set may be {@code null}.
     */
    public PathToConversionCriteria setConversionDimensions(java.util.List<SortedDimension> conversionDimensions) {
      this.conversionDimensions = conversionDimensions;
      return this;
    }

    /**
     * The list of custom floodlight variables the report should include.
     * The value returned may be {@code null}.
     */
    public java.util.List<SortedDimension> getCustomFloodlightVariables() {
      return customFloodlightVariables;
    }

    /**
     * The list of custom floodlight variables the report should include.
     * The value set may be {@code null}.
     */
    public PathToConversionCriteria setCustomFloodlightVariables(java.util.List<SortedDimension> customFloodlightVariables) {
      this.customFloodlightVariables = customFloodlightVariables;
      return this;
    }

    /**
     * The date range this report should be run for.
     * The value returned may be {@code null}.
     */
    public DateRange getDateRange() {
      return dateRange;
    }

    /**
     * The date range this report should be run for.
     * The value set may be {@code null}.
     */
    public PathToConversionCriteria setDateRange(DateRange dateRange) {
      this.dateRange = dateRange;
      return this;
    }

    /**
     * The floodlight ID for which to show data in this report. All advertisers associated with that
     * ID will automatically be added. The dimension of the value needs to be
     * 'dfa:floodlightConfigId'.
     * The value returned may be {@code null}.
     */
    public DimensionValue getFloodlightConfigId() {
      return floodlightConfigId;
    }

    /**
     * The floodlight ID for which to show data in this report. All advertisers associated with that
     * ID will automatically be added. The dimension of the value needs to be
     * 'dfa:floodlightConfigId'.
     * The value set may be {@code null}.
     */
    public PathToConversionCriteria setFloodlightConfigId(DimensionValue floodlightConfigId) {
      this.floodlightConfigId = floodlightConfigId;
      return this;
    }

    /**
     * The list of names of metrics the report should include.
     * The value returned may be {@code null}.
     */
    public java.util.List<String> getMetricNames() {
      return metricNames;
    }

    /**
     * The list of names of metrics the report should include.
     * The value set may be {@code null}.
     */
    public PathToConversionCriteria setMetricNames(java.util.List<String> metricNames) {
      this.metricNames = metricNames;
      return this;
    }

    /**
     * The list of per interaction dimensions the report should include.
     * The value returned may be {@code null}.
     */
    public java.util.List<SortedDimension> getPerInteractionDimensions() {
      return perInteractionDimensions;
    }

    /**
     * The list of per interaction dimensions the report should include.
     * The value set may be {@code null}.
     */
    public PathToConversionCriteria setPerInteractionDimensions(java.util.List<SortedDimension> perInteractionDimensions) {
      this.perInteractionDimensions = perInteractionDimensions;
      return this;
    }

    /**
     * The properties of the report.
     * The value returned may be {@code null}.
     */
    public ReportProperties getReportProperties() {
      return reportProperties;
    }

    /**
     * The properties of the report.
     * The value set may be {@code null}.
     */
    public PathToConversionCriteria setReportProperties(ReportProperties reportProperties) {
      this.reportProperties = reportProperties;
      return this;
    }

    /**
     * The properties of the report.
     */
    public static final class ReportProperties extends GenericJson {

      /**
       * DFA checks to see if a click interaction occurred within the specified period of time before a
       * conversion. By default the value is pulled from Floodlight or you can manually enter a custom
       * value. Valid values: 1-90.
       * The value may be {@code null}.
       */
      @com.google.api.client.util.Key
      private Integer clicksLookbackWindow;

      /**
       * DFA checks to see if an impression interaction occurred within the specified period of time
       * before a conversion. By default the value is pulled from Floodlight or you can manually enter a
       * custom value. Valid values: 1-90.
       * The value may be {@code null}.
       */
      @com.google.api.client.util.Key
      private Integer impressionsLookbackWindow;

      /**
       * Include conversions that have no cookie, but do have an exposure path.
       * The value may be {@code null}.
       */
      @com.google.api.client.util.Key
      private Boolean includeAttributedIPConversions;

      /**
       * Include conversions of users with a DoubleClick cookie but without an exposure. That means the
       * user did not click or see an ad from the advertiser within the Floodlight group, or that the
       * interaction happened outside the lookback window.
       * The value may be {@code null}.
       */
      @com.google.api.client.util.Key
      private Boolean includeUnattributedCookieConversions;

      /**
       * Include conversions that have no associated cookies and no exposures. It’s therefore impossible
       * to know how the user was exposed to your ads during the lookback window prior to a conversion.
       * The value may be {@code null}.
       */
      @com.google.api.client.util.Key
      private Boolean includeUnattributedIPConversions;

      /**
       * The maximum number of click interactions to include in the report. Advertisers currently paying
       * for E2C reports get up to 200 (100 clicks, 100 impressions). If another advertiser in your
       * network is paying for E2C, you can have up to 5 total exposures per report.
       * The value may be {@code null}.
       */
      @com.google.api.client.util.Key
      private Integer maximumClickInteractions;

      /**
       * The maximum number of click interactions to include in the report. Advertisers currently paying
       * for E2C reports get up to 200 (100 clicks, 100 impressions). If another advertiser in your
       * network is paying for E2C, you can have up to 5 total exposures per report.
       * The value may be {@code null}.
       */
      @com.google.api.client.util.Key
      private Integer maximumImpressionInteractions;

      /**
       * The maximum amount of time that can take place between interactions (clicks or impressions) by
       * the same user. Valid values: 1-90.
       * The value may be {@code null}.
       */
      @com.google.api.client.util.Key
      private Integer maximumInteractionGap;

      /**
       * Enable pivoting on interaction path.
       * The value may be {@code null}.
       */
      @com.google.api.client.util.Key
      private Boolean pivotOnInteractionPath;

      /**
       * DFA checks to see if a click interaction occurred within the specified period of time before a
       * conversion. By default the value is pulled from Floodlight or you can manually enter a custom
       * value. Valid values: 1-90.
       * The value returned may be {@code null}.
       */
      public Integer getClicksLookbackWindow() {
        return clicksLookbackWindow;
      }

      /**
       * DFA checks to see if a click interaction occurred within the specified period of time before a
       * conversion. By default the value is pulled from Floodlight or you can manually enter a custom
       * value. Valid values: 1-90.
       * The value set may be {@code null}.
       */
      public ReportProperties setClicksLookbackWindow(Integer clicksLookbackWindow) {
        this.clicksLookbackWindow = clicksLookbackWindow;
        return this;
      }

      /**
       * DFA checks to see if an impression interaction occurred within the specified period of time
       * before a conversion. By default the value is pulled from Floodlight or you can manually enter a
       * custom value. Valid values: 1-90.
       * The value returned may be {@code null}.
       */
      public Integer getImpressionsLookbackWindow() {
        return impressionsLookbackWindow;
      }

      /**
       * DFA checks to see if an impression interaction occurred within the specified period of time
       * before a conversion. By default the value is pulled from Floodlight or you can manually enter a
       * custom value. Valid values: 1-90.
       * The value set may be {@code null}.
       */
      public ReportProperties setImpressionsLookbackWindow(Integer impressionsLookbackWindow) {
        this.impressionsLookbackWindow = impressionsLookbackWindow;
        return this;
      }

      /**
       * Include conversions that have no cookie, but do have an exposure path.
       * The value returned may be {@code null}.
       */
      public Boolean getIncludeAttributedIPConversions() {
        return includeAttributedIPConversions;
      }

      /**
       * Include conversions that have no cookie, but do have an exposure path.
       * The value set may be {@code null}.
       */
      public ReportProperties setIncludeAttributedIPConversions(Boolean includeAttributedIPConversions) {
        this.includeAttributedIPConversions = includeAttributedIPConversions;
        return this;
      }

      /**
       * Include conversions of users with a DoubleClick cookie but without an exposure. That means the
       * user did not click or see an ad from the advertiser within the Floodlight group, or that the
       * interaction happened outside the lookback window.
       * The value returned may be {@code null}.
       */
      public Boolean getIncludeUnattributedCookieConversions() {
        return includeUnattributedCookieConversions;
      }

      /**
       * Include conversions of users with a DoubleClick cookie but without an exposure. That means the
       * user did not click or see an ad from the advertiser within the Floodlight group, or that the
       * interaction happened outside the lookback window.
       * The value set may be {@code null}.
       */
      public ReportProperties setIncludeUnattributedCookieConversions(Boolean includeUnattributedCookieConversions) {
        this.includeUnattributedCookieConversions = includeUnattributedCookieConversions;
        return this;
      }

      /**
       * Include conversions that have no associated cookies and no exposures. It’s therefore impossible
       * to know how the user was exposed to your ads during the lookback window prior to a conversion.
       * The value returned may be {@code null}.
       */
      public Boolean getIncludeUnattributedIPConversions() {
        return includeUnattributedIPConversions;
      }

      /**
       * Include conversions that have no associated cookies and no exposures. It’s therefore impossible
       * to know how the user was exposed to your ads during the lookback window prior to a conversion.
       * The value set may be {@code null}.
       */
      public ReportProperties setIncludeUnattributedIPConversions(Boolean includeUnattributedIPConversions) {
        this.includeUnattributedIPConversions = includeUnattributedIPConversions;
        return this;
      }

      /**
       * The maximum number of click interactions to include in the report. Advertisers currently paying
       * for E2C reports get up to 200 (100 clicks, 100 impressions). If another advertiser in your
       * network is paying for E2C, you can have up to 5 total exposures per report.
       * The value returned may be {@code null}.
       */
      public Integer getMaximumClickInteractions() {
        return maximumClickInteractions;
      }

      /**
       * The maximum number of click interactions to include in the report. Advertisers currently paying
       * for E2C reports get up to 200 (100 clicks, 100 impressions). If another advertiser in your
       * network is paying for E2C, you can have up to 5 total exposures per report.
       * The value set may be {@code null}.
       */
      public ReportProperties setMaximumClickInteractions(Integer maximumClickInteractions) {
        this.maximumClickInteractions = maximumClickInteractions;
        return this;
      }

      /**
       * The maximum number of click interactions to include in the report. Advertisers currently paying
       * for E2C reports get up to 200 (100 clicks, 100 impressions). If another advertiser in your
       * network is paying for E2C, you can have up to 5 total exposures per report.
       * The value returned may be {@code null}.
       */
      public Integer getMaximumImpressionInteractions() {
        return maximumImpressionInteractions;
      }

      /**
       * The maximum number of click interactions to include in the report. Advertisers currently paying
       * for E2C reports get up to 200 (100 clicks, 100 impressions). If another advertiser in your
       * network is paying for E2C, you can have up to 5 total exposures per report.
       * The value set may be {@code null}.
       */
      public ReportProperties setMaximumImpressionInteractions(Integer maximumImpressionInteractions) {
        this.maximumImpressionInteractions = maximumImpressionInteractions;
        return this;
      }

      /**
       * The maximum amount of time that can take place between interactions (clicks or impressions) by
       * the same user. Valid values: 1-90.
       * The value returned may be {@code null}.
       */
      public Integer getMaximumInteractionGap() {
        return maximumInteractionGap;
      }

      /**
       * The maximum amount of time that can take place between interactions (clicks or impressions) by
       * the same user. Valid values: 1-90.
       * The value set may be {@code null}.
       */
      public ReportProperties setMaximumInteractionGap(Integer maximumInteractionGap) {
        this.maximumInteractionGap = maximumInteractionGap;
        return this;
      }

      /**
       * Enable pivoting on interaction path.
       * The value returned may be {@code null}.
       */
      public Boolean getPivotOnInteractionPath() {
        return pivotOnInteractionPath;
      }

      /**
       * Enable pivoting on interaction path.
       * The value set may be {@code null}.
       */
      public ReportProperties setPivotOnInteractionPath(Boolean pivotOnInteractionPath) {
        this.pivotOnInteractionPath = pivotOnInteractionPath;
        return this;
      }

    }
  }

  /**
   * The report criteria for a report of type "REACH".
   */
  public static final class ReachCriteria extends GenericJson {

    /**
     * Activity group.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private Activities activities;

    /**
     * Custom Rich Media Events group.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private CustomRichMediaEvents customRichMediaEvents;

    /**
     * The date range this report should be run for.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private DateRange dateRange;

    /**
     * The list of filters on which dimensions are filtered. Filters for different dimensions are
     * ANDed, filters for the same dimension are grouped together and ORed.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private java.util.List<DimensionValue> dimensionFilters;

    static {
      // hack to force ProGuard to consider DimensionValue used, since otherwise it would be stripped out
      // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
      com.google.api.client.util.Data.nullOf(DimensionValue.class);
    }

    /**
     * The list of dimensions the report should include.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private java.util.List<SortedDimension> dimensions;

    static {
      // hack to force ProGuard to consider SortedDimension used, since otherwise it would be stripped out
      // see http://code.google.com/p/google-api-java-client/issues/detail?id=528
      com.google.api.client.util.Data.nullOf(SortedDimension.class);
    }

    /**
     * The list of names of metrics the report should include.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private java.util.List<String> metricNames;

    /**
     * The list of names of  Reach By Frequency metrics the report should include.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private java.util.List<String> reachByFrequencyMetricNames;

    /**
     * Activity group.
     * The value returned may be {@code null}.
     */
    public Activities getActivities() {
      return activities;
    }

    /**
     * Activity group.
     * The value set may be {@code null}.
     */
    public ReachCriteria setActivities(Activities activities) {
      this.activities = activities;
      return this;
    }

    /**
     * Custom Rich Media Events group.
     * The value returned may be {@code null}.
     */
    public CustomRichMediaEvents getCustomRichMediaEvents() {
      return customRichMediaEvents;
    }

    /**
     * Custom Rich Media Events group.
     * The value set may be {@code null}.
     */
    public ReachCriteria setCustomRichMediaEvents(CustomRichMediaEvents customRichMediaEvents) {
      this.customRichMediaEvents = customRichMediaEvents;
      return this;
    }

    /**
     * The date range this report should be run for.
     * The value returned may be {@code null}.
     */
    public DateRange getDateRange() {
      return dateRange;
    }

    /**
     * The date range this report should be run for.
     * The value set may be {@code null}.
     */
    public ReachCriteria setDateRange(DateRange dateRange) {
      this.dateRange = dateRange;
      return this;
    }

    /**
     * The list of filters on which dimensions are filtered. Filters for different dimensions are
     * ANDed, filters for the same dimension are grouped together and ORed.
     * The value returned may be {@code null}.
     */
    public java.util.List<DimensionValue> getDimensionFilters() {
      return dimensionFilters;
    }

    /**
     * The list of filters on which dimensions are filtered. Filters for different dimensions are
     * ANDed, filters for the same dimension are grouped together and ORed.
     * The value set may be {@code null}.
     */
    public ReachCriteria setDimensionFilters(java.util.List<DimensionValue> dimensionFilters) {
      this.dimensionFilters = dimensionFilters;
      return this;
    }

    /**
     * The list of dimensions the report should include.
     * The value returned may be {@code null}.
     */
    public java.util.List<SortedDimension> getDimensions() {
      return dimensions;
    }

    /**
     * The list of dimensions the report should include.
     * The value set may be {@code null}.
     */
    public ReachCriteria setDimensions(java.util.List<SortedDimension> dimensions) {
      this.dimensions = dimensions;
      return this;
    }

    /**
     * The list of names of metrics the report should include.
     * The value returned may be {@code null}.
     */
    public java.util.List<String> getMetricNames() {
      return metricNames;
    }

    /**
     * The list of names of metrics the report should include.
     * The value set may be {@code null}.
     */
    public ReachCriteria setMetricNames(java.util.List<String> metricNames) {
      this.metricNames = metricNames;
      return this;
    }

    /**
     * The list of names of  Reach By Frequency metrics the report should include.
     * The value returned may be {@code null}.
     */
    public java.util.List<String> getReachByFrequencyMetricNames() {
      return reachByFrequencyMetricNames;
    }

    /**
     * The list of names of  Reach By Frequency metrics the report should include.
     * The value set may be {@code null}.
     */
    public ReachCriteria setReachByFrequencyMetricNames(java.util.List<String> reachByFrequencyMetricNames) {
      this.reachByFrequencyMetricNames = reachByFrequencyMetricNames;
      return this;
    }

  }

  /**
   * The report's schedule. Can only be set if the report's 'dateRange' is a relative date range and
   * the relative date range is not "TODAY".
   */
  public static final class Schedule extends GenericJson {

    /**
     * Whether the schedule is active or not. Must be set to either true or false.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private Boolean active;

    /**
     * Defines every how many days, weeks or months the report should be run. Needs to be set when
     * "repeats" is either "DAILY", "WEEKLY" or "MONTHLY".
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private Integer every;

    /**
     * The expiration date when the scheduled report stops running.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private DateTime expirationDate;

    /**
     * The interval for which the report is repeated, one of: - "DAILY", also requires field "every"
     * to be set.  - "WEEKLY", also requires fields "every" and "repeatsOnWeekDays" to be set.  -
     * "TWICE_A_MONTH"  - "MONTHLY", also requires fields "every" and "runsOnDayOfMonth" to be set.  -
     * "QUARTERLY"  - "YEARLY"
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String repeats;

    /**
     * List of week days "WEEKLY" on which scheduled reports should run.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private java.util.List<String> repeatsOnWeekDays;

    /**
     * Enum to define for "MONTHLY" scheduled reports whether reports should be repeated on the same
     * day of the month as "startDate" or the same day of the week of the month. Possible values are:
     * - DAY_OF_MONTH  - WEEK_OF_MONTH   Example: If 'startDate' is Monday, April 2nd 2012
     * (2012-04-02), "DAY_OF_MONTH" would run subsequent reports on the 2nd of every Month, and
     * "WEEK_OF_MONTH" would run subsequent reports on the first Monday of the month.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private String runsOnDayOfMonth;

    /**
     * Start date of date range for which scheduled reports should be run.
     * The value may be {@code null}.
     */
    @com.google.api.client.util.Key
    private DateTime startDate;

    /**
     * Whether the schedule is active or not. Must be set to either true or false.
     * The value returned may be {@code null}.
     */
    public Boolean getActive() {
      return active;
    }

    /**
     * Whether the schedule is active or not. Must be set to either true or false.
     * The value set may be {@code null}.
     */
    public Schedule setActive(Boolean active) {
      this.active = active;
      return this;
    }

    /**
     * Defines every how many days, weeks or months the report should be run. Needs to be set when
     * "repeats" is either "DAILY", "WEEKLY" or "MONTHLY".
     * The value returned may be {@code null}.
     */
    public Integer getEvery() {
      return every;
    }

    /**
     * Defines every how many days, weeks or months the report should be run. Needs to be set when
     * "repeats" is either "DAILY", "WEEKLY" or "MONTHLY".
     * The value set may be {@code null}.
     */
    public Schedule setEvery(Integer every) {
      this.every = every;
      return this;
    }

    /**
     * The expiration date when the scheduled report stops running.
     * The value returned may be {@code null}.
     */
    public DateTime getExpirationDate() {
      return expirationDate;
    }

    /**
     * The expiration date when the scheduled report stops running.
     * The value set may be {@code null}.
     */
    public Schedule setExpirationDate(DateTime expirationDate) {
      this.expirationDate = expirationDate;
      return this;
    }

    /**
     * The interval for which the report is repeated, one of: - "DAILY", also requires field "every"
     * to be set.  - "WEEKLY", also requires fields "every" and "repeatsOnWeekDays" to be set.  -
     * "TWICE_A_MONTH"  - "MONTHLY", also requires fields "every" and "runsOnDayOfMonth" to be set.  -
     * "QUARTERLY"  - "YEARLY"
     * The value returned may be {@code null}.
     */
    public String getRepeats() {
      return repeats;
    }

    /**
     * The interval for which the report is repeated, one of: - "DAILY", also requires field "every"
     * to be set.  - "WEEKLY", also requires fields "every" and "repeatsOnWeekDays" to be set.  -
     * "TWICE_A_MONTH"  - "MONTHLY", also requires fields "every" and "runsOnDayOfMonth" to be set.  -
     * "QUARTERLY"  - "YEARLY"
     * The value set may be {@code null}.
     */
    public Schedule setRepeats(String repeats) {
      this.repeats = repeats;
      return this;
    }

    /**
     * List of week days "WEEKLY" on which scheduled reports should run.
     * The value returned may be {@code null}.
     */
    public java.util.List<String> getRepeatsOnWeekDays() {
      return repeatsOnWeekDays;
    }

    /**
     * List of week days "WEEKLY" on which scheduled reports should run.
     * The value set may be {@code null}.
     */
    public Schedule setRepeatsOnWeekDays(java.util.List<String> repeatsOnWeekDays) {
      this.repeatsOnWeekDays = repeatsOnWeekDays;
      return this;
    }

    /**
     * Enum to define for "MONTHLY" scheduled reports whether reports should be repeated on the same
     * day of the month as "startDate" or the same day of the week of the month. Possible values are:
     * - DAY_OF_MONTH  - WEEK_OF_MONTH   Example: If 'startDate' is Monday, April 2nd 2012
     * (2012-04-02), "DAY_OF_MONTH" would run subsequent reports on the 2nd of every Month, and
     * "WEEK_OF_MONTH" would run subsequent reports on the first Monday of the month.
     * The value returned may be {@code null}.
     */
    public String getRunsOnDayOfMonth() {
      return runsOnDayOfMonth;
    }

    /**
     * Enum to define for "MONTHLY" scheduled reports whether reports should be repeated on the same
     * day of the month as "startDate" or the same day of the week of the month. Possible values are:
     * - DAY_OF_MONTH  - WEEK_OF_MONTH   Example: If 'startDate' is Monday, April 2nd 2012
     * (2012-04-02), "DAY_OF_MONTH" would run subsequent reports on the 2nd of every Month, and
     * "WEEK_OF_MONTH" would run subsequent reports on the first Monday of the month.
     * The value set may be {@code null}.
     */
    public Schedule setRunsOnDayOfMonth(String runsOnDayOfMonth) {
      this.runsOnDayOfMonth = runsOnDayOfMonth;
      return this;
    }

    /**
     * Start date of date range for which scheduled reports should be run.
     * The value returned may be {@code null}.
     */
    public DateTime getStartDate() {
      return startDate;
    }

    /**
     * Start date of date range for which scheduled reports should be run.
     * The value set may be {@code null}.
     */
    public Schedule setStartDate(DateTime startDate) {
      this.startDate = startDate;
      return this;
    }

  }

}
